package net.wanji.datacenter.cache;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.databus.dao.mapper.CrossDirDataHistMapper;
import net.wanji.databus.po.CrossDirDataHistPO;
import net.wanji.datacenter.common.Constants;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.temporal.ChronoUnit;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/10/18 15:50
 */
@Component
@Slf4j
public class CrossDirWeekMaxFlowCache implements CommandLineRunner {

    public static final Map<String, Integer> crossDirWeekMaxFlowMap = new HashMap<>();

    @Resource
    private CrossDirDataHistMapper crossDirDataHistMapper;

    private void getMaxFlow() throws Exception {
        LocalDate currentDate = LocalDate.now();
        LocalDate lastDay = currentDate.minus(1, ChronoUnit.DAYS);
        LocalDate lastWeek = currentDate.minus(1, ChronoUnit.WEEKS);
        List<String> crossIds = CrossInfoCache.getSignalIds();
        for (String crossId : crossIds) {
            // 前一周
            LambdaQueryWrapper<CrossDirDataHistPO> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(CrossDirDataHistPO::getCrossId, crossId);
            queryWrapper.between(CrossDirDataHistPO::getStartTime, lastWeek, lastDay);
            List<CrossDirDataHistPO> crossDirDataHistPOS = crossDirDataHistMapper.selectList(queryWrapper);

            if (!CollectionUtils.isEmpty(crossDirDataHistPOS)) {
                Map<Integer, List<CrossDirDataHistPO>> dirDataMap = crossDirDataHistPOS.stream().collect(Collectors.groupingBy(CrossDirDataHistPO::getDirType));
                if (!dirDataMap.isEmpty()) {
                    for (Map.Entry<Integer, List<CrossDirDataHistPO>> entry : dirDataMap.entrySet()) {
                        Integer dir = entry.getKey();
                        List<CrossDirDataHistPO> dirDataHistPOS = entry.getValue();
                        String currentCrossId = dirDataHistPOS.get(0).getCrossId();
                        int dirFlowMax = dirDataHistPOS.stream().map(CrossDirDataHistPO::getFlow).mapToInt(Integer::intValue).max().getAsInt();
                        crossDirWeekMaxFlowMap.put(currentCrossId.concat(Constants.underline).concat(String.valueOf(dir)), dirFlowMax);
                    }
                }
            }
        }
    }

    @Override
    public void run(String... args) throws Exception {
        try {
            getMaxFlow();
        } catch (Exception e) {
            log.error("7天内路口方向最大流量获取失败", e);
            throw new Exception(e);
        }
        log.info("7天内路口方向最大流量：{}", crossDirWeekMaxFlowMap);
    }

}