package net.wanji.datacenter.cache;

import lombok.extern.slf4j.Slf4j;
import net.wanji.databus.dao.mapper.LaneInfoMapper;
import net.wanji.databus.po.LaneInfoPO;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2023/10/25 18:54
 */
@Component
@Slf4j
public class CrossLaneInfoCache implements InitializingBean {

    public static final Map<String, LaneInfoPO> laneInfoMap = new HashMap<>();

    @Resource
    private LaneInfoMapper laneInfoMapper;

    private void init() throws Exception {
        List<String> crossIds = CrossInfoCache.getSignalIds();
        for (String crossId : crossIds) {
            List<LaneInfoPO> laneInfoPOS = laneInfoMapper.selectBycrossId(crossId);
            if (!CollectionUtils.isEmpty(laneInfoPOS)) {
                Map<String, LaneInfoPO> crossMap = laneInfoPOS.stream().collect(Collectors.toMap(LaneInfoPO::getId, Function.identity(), (key1, key2) -> key2));
                laneInfoMap.putAll(crossMap);
            }
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        try {
            init();
        } catch (Exception e) {
            log.error("信控路口车道信息加载失败", e);
            throw new Exception(e);
        }
        log.info("信控路口车道信息加载成功");
    }
}
